/*
 * FILENAME
 *   isr_short.c
 *
 * DESCRIPTION
 *   ISR for block processing, provides a working buffer of type short.
 *   Note, different ISR versions can be found on our webpage: 
 *     www.spsc.tugraz.at/courses/dsplab/
 *
 *   Rev 5.0  Aug. 30, 2005    C.Feldbauer <feldbauer@tugraz.at>
 *   Rev 4.0  Nov. 14, 2002    C.Feldbauer <feldbauer@tugraz.at>
 *   Rev 3.2  Nov. 09, 2001    C.Feldbauer <feldbauer@tugraz.at>
 *   Rev 3.1  Oct. 30, 2001    C.Feldbauer <feldbauer@tugraz.at>
 */


#include "blocks.h"  // definition of block size, block overlap, ...





// global variables (defined in main.c)
extern int dma_index;
extern short in[], out[];


// the working buffer (for processing of one channel's data)
short workbuf[BLKSZ];
#pragma DATA_ALIGN(workbuf, WBALIGN); 


/**********************  I S R  *********************/
/******* Interrupts from EDMA are handled here ******/
/* DON'T forget to add ISR vectors to your project. */

interrupt void edma_isr( void)
{
  int i, p, offset;
  
  // clear edma CIPR bit 8
  EDMA_intClear(8);
  
  // circular buffer management...
  // next edma frame (set automatically via reload params)
  if( dma_index == NBLOCKS-1)
    dma_index = 0;
  else
    dma_index++;
  
  offset = dma_index * NCHAN*HOPSZ;  
  /* the interesting part of the block processing begins here...     
     for the case 'OVERLAP > 0', 'offset' points to the OLDEST data!
  */
  
  
  /* Unwrap incoming block and extract only left (?) audio channel.
     For the case 'OVERLAP > 0', we get workbuf[]={OLDEST, ...., NEWEST}.
     Note, when we can assure that 'processing()' can deal with a circular buffer, 
     the unwrapping is not needed.
   */
  for( i=0, p=offset;  i < BLKSZ;  i++, p=(p==NCHAN*NBLOCKS*HOPSZ-NCHAN)?0:p+NCHAN )
    workbuf[i] = in[p];
  
  
  /* the actual signal processing is done in 'processing()'
     using 'workbuf' as input AND output
  */
  processing( (void *)workbuf);
  
  
  
  // write processed data to dma buffers (left & right channel) ...  
  i = 0;
  p = offset;

#if OLA != 0
#if OVERLAP != 0
  // add overlapping parts
  for( ;  i < BLKSZ-HOPSZ;  i++, p=(p==NCHAN*NBLOCKS*HOPSZ-NCHAN)?0:p+NCHAN )
  {
    out[p] = out[p] + workbuf[i];
    out[p+1] = out[p+1] + workbuf[i];
  }  
#endif
  
  // write non-overlapping part to DMA buffer
  for( ;  i < BLKSZ;        i++, p=(p==NCHAN*NBLOCKS*HOPSZ-NCHAN)?0:p+NCHAN )
  {
    out[p] = workbuf[i];
    out[p+1] = workbuf[i];
  }
#else
  // take the first block if OLA==0
  for( ;  i < HOPSZ;        i++, p=(p==NCHAN*NBLOCKS*HOPSZ-NCHAN)?0:p+NCHAN )
  {
    out[p] = workbuf[i];
    out[p+1] = workbuf[i];
  }
#endif
}
